from ellipsis import apiManager
from ellipsis import sanitize


def add( name, token, parentId = None, publicAccess =None, metadata=None):
    name = sanitize.validString('name', name, True)
    token = sanitize.validString('token', token, True)
    parentId = sanitize.validUuid('parentId', parentId, False)
    metadata = sanitize.validObject('metadata', metadata, False)
    publicAccess = sanitize.validObject('publicAccess', publicAccess, False)

    body = {'name': name, 'parentId':parentId, 'publicAccess':publicAccess, 'metadata':metadata }

    return apiManager.post('/path/pointCloud', body, token)





from ellipsis import apiManager, sanitize

def get(pathId, token=None, memberType=['inherited','direct']):
    pathId = sanitize.validUuid('pathId', pathId, True)
    token = sanitize.validString('token', token, False)
    memberType = sanitize.validStringArray('memberType', memberType, False)

    body = {
        'type': memberType
    }
    
    return apiManager.get(f'/path/{pathId}/member', body, token)
    

def delete(pathId, userId, token):
    pathId = sanitize.validUuid('pathId', pathId, True)
    token = sanitize.validString('token', token, True)
    userId = sanitize.validUuid('userId', userId, True)

    return apiManager.delete(f'/path/{pathId}/member/{userId}', None, token)


def edit(pathId, userId, access, token):
    pathId = sanitize.validUuid('pathId', pathId, True)
    token = sanitize.validString('token', token, True)
    userId = sanitize.validUuid('userId', userId, True)
    access = sanitize.validObject('access', access, True)

    return apiManager.patch(f'/path/{pathId}/member/{userId}', {
        'access': access
    }, token)



from ellipsis import apiManager
from ellipsis import sanitize
import os
from ellipsis.util.root import recurse
import numpy as np
import geopandas as gpd

def add(pathId, timestampId, token, fileFormat, filePath=None, memFile = None, name = None, epsg = None, dateColumns = None, datePatterns = None, method= 'simplify', fastUpload = True):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True) 
    timestampId = sanitize.validUuid('timestampId', timestampId, True) 
    filePath = sanitize.validString('filePath', filePath, False)
    epsg = sanitize.validInt('epsg', epsg, False)
    fileFormat = sanitize.validString('fileFormat', fileFormat, True)
    dateColumns = sanitize.validStringArray('dateColumns', dateColumns, False)    
    datePatterns = sanitize.validStringArray('datePatterns', datePatterns, False)    
    name = sanitize.validString('name', name, False)
    fastUpload = sanitize.validBool('fastUpload', fastUpload, True)
    if fastUpload:
        fastUpload='true'
    else:
        fastUpload = 'false'


    if type(memFile) == type(None) and type(filePath) == type(None):
        raise ValueError('You need to specify either a filePath or a memFile')

    if type(memFile) != type(None) and type(name) == type(None):
        raise ValueError('Parameter name is required when using a memory file')

    if type(name ) == type(None):
        seperator = os.path.sep
        fileName = filePath.split(seperator)[len(filePath.split(seperator))-1 ]
    else:
        fileName = name


    body = {'name':fileName, 'epsg':epsg, 'format':fileFormat, 'dateColumns': dateColumns, 'datePatterns':datePatterns, 'fastUpload':fastUpload}
    if type(memFile) == type(None):
        r = apiManager.upload('/path/' + pathId + '/vector/timestamp/' + timestampId + '/file' , filePath, body, token)
    else:
        r = apiManager.upload('/path/' + pathId + '/vector/timestamp/' + timestampId + '/file', name, body, token,
                              memfile=memFile)

    return r

def get(pathId, timestampId, token, pageStart = None, listAll = True):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True) 
    timestampId = sanitize.validUuid('timestampId', timestampId, True) 
    pageStart = sanitize.validUuid('pageStart', pageStart, False)
    listAll = sanitize.validBool('listAll', listAll, True)

    body = {'pageStart': pageStart}
    def f(body):    
        r = apiManager.get('/path/' + pathId + '/vector/timestamp/' + timestampId + '/upload', body, token)    
        
        for i in np.arange( len(r['result'])):
            if 'info' in r['result'][i].keys() and 'bounds' in r['result'][i]['info'].keys() and type(r['result'][i]['info']['bounds']) != type(None):
                r['result'][i]['info']['bounds'] = gpd.GeoDataFrame.from_features([{'id': 0, 'properties':{}, 'geometry':r['result'][i]['info']['bounds']}]).unary_union

        return r

    r = recurse(f, body, listAll)
    return r

def download(pathId, timestampId, fileId, filePath, token):
    
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True) 
    fileId = sanitize.validUuid('fileId', fileId, True) 
    timestampId = sanitize.validUuid('timestampId', timestampId, True) 
    filePath = sanitize.validString('filePath', filePath, True)


    apiManager.download('/path/' + pathId + '/vector/timestamp/' + timestampId + '/file/' + fileId + '/data', filePath, token)



def revert(pathId, timestampId, fileId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True) 
    timestampId = sanitize.validUuid('timestampId', timestampId, True) 


    
    body = {'revert':True}
    r = apiManager.post('/path/' + pathId + '/vector/timestamp/' + timestampId + '/file/' + fileId + '/revert' ,  body, token)
    return r


def recover(pathId, timestampId, fileId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True) 
    timestampId = sanitize.validUuid('timestampId', timestampId, True) 


    
    body = {'revert':False}
    r = apiManager.post('/path/' + pathId + '/vector/timestamp/' + timestampId + '/file/' + fileId + '/revert' ,  body, token)
    return r



def delete(pathId, timestampId, fileId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True) 
    timestampId = sanitize.validUuid('timestampId', timestampId, True) 


    
    r = apiManager.delete('/path/' + pathId + '/vector/timestamp/' + timestampId + '/file/' + fileId  , None, token)
    return r





from ellipsis import apiManager
from ellipsis import sanitize
import os

def get(token):
    token = sanitize.validString('token', token, False)

    r = apiManager.get('/path/setUpTask', {}, token)
        
    return r


def copernicusImport( parameters, token):
    token = sanitize.validString('token', token, True)
    parameters = sanitize.validObject('parameters', parameters, True)
    r = apiManager.post('/path/setUpTask', {'type':'copernicusImport', 'parameters':parameters}, token)
        
    return r


def fileImport(  token, memFile=None, parentId = None, filePath=None, fileFormat = 'geopackage', name = None):
    token = sanitize.validString('token', token, True)
    parentId = sanitize.validUuid(('parentId', parentId, False))
    filePath = sanitize.validString('filePath', filePath, False)
    name = sanitize.validString('name', name, False)
    fileFormat = sanitize.validString('fileFormat', fileFormat, True)


    if type(memFile) == type(None) and type(filePath) == type(None):
        raise ValueError('You need to specify either a filePath or a memFile')
    if type(memFile) != type(None) and type(name) == type(None):
        raise ValueError('Parameter name is required when using a memory file')

    if type(name) != type(None):
        nameToUse = name
    else:
        seperator = os.path.sep
        nameToUse = filePath.split(seperator)[len(filePath.split(seperator)) - 1]

    parameters = {'name': nameToUse, 'parentId': parentId}
    body = {'type': fileFormat + 'Import', 'parameters': parameters}
    if type(memFile) == type(None):
        r = apiManager.post('/path/setUpTask', body, token)
    else:
        r = apiManager.upload('/path/setUpTask' , filePath , name, body, token, memfile = memFile)
    return r







import setuptools
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ellipsis",
    version="3.1.48",
    author="Daniel van der Maas",
    author_email="daniel@ellipsis-drive.com",
    description="Package to interact with the Ellipsis API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ellipsis-drive-internal/python-package",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    install_requires=[
    'pandas',
    'Pillow',
    'matplotlib',
    'geopandas',
    'pyproj',
    'imageio',
    'numpy',
    'imagecodecs',
    'requests',
    'requests-toolbelt',
    'rasterio',
    'Shapely',
    'geopy',
    'scikit-image',
    'Fiona',
    'pygltflib',
    'tifffile'
    ],
    python_requires='>=3.6',
)



from uuid import UUID
import numpy as np
import datetime
import json
import geopandas as gpd
import pandas as pd
from shapely import geometry

from ellipsis.util.root import reprojectVector


def validUuid(name, value, required):
    if not required and type(value) == type(None):
        return
    try:
        UUID(value, version=2)
    except:
        raise ValueError(name + ' must be of type string and be a uuid')
    return(value)

def validResolution(name, value, required):
    if not required and type(value) == type(None):
        return

    if type(value) != type({}):
        raise ValueError(name + ' must be of type dictionary with properties min and max as float')

    if 'min' not in value.keys() or 'max' not in value.keys():
        raise ValueError(name + ' must be of type dictionary with properties min and max as float')

    try:
        value = {'min': float(value['min']), 'max':float(value['max']) }        
    except:
        raise ValueError(name + ' must be of type dictionary with properties min and max as float')

    return(value)



def validDateRange(name, value, required):

    if not required and type(value) == type(None):
        return
    
    
    if  ( type(value) != type({}) or 'from' not in value.keys() or 'to' not in value.keys()): 
        raise ValueError('date must be a dictionary with properties from and to that should be of type datetime')

    dateFrom = value['from']
    dateTo = value['to']
    dateFrom = validDate('dateFrom', dateFrom, False)    
    dateTo = validDate('dateTo', dateTo, False)    
    return {'from':dateFrom, 'to':dateTo}

def validUuidArray(name, value, required):
    if not required and type(value) == type(None):
        return

    try:
        value = list(value)        
    except:
        raise ValueError(name + ' must be an iterable')

    for uuid in value:
        try:
            UUID(uuid, version=2)
        except:
            raise ValueError(name + ' must be a list of uuids')    
    return(value)

def validPandas(name, value, required):
    if not required and type(value) == type(None):
        return

    if type(value) != type(pd.DataFrame()):
        raise ValueError(name + 'must be of type pandas data frame')
    return(value)

def validString(name, value, required):
    if not required and type(value) == type(None):
        return

    if type(value) != type('x'):
        raise ValueError(name + 'must be of type string')
    return(value)

def validShapely(name, value, required):
    if not required and type(value) == type(None):
        return

    if not 'shapely' in str(type(value)):
        raise ValueError(name + 'must be a shapely geometry')
    return(value)


def validBoundsCrsCombination(name, value, required):
    if not required and type(value) == type(None):
        return

    bounds = value[0]
    crs = value[1]
    
    value = bounds
    keys = ['xMin', 'xMax', 'yMin', 'yMax']
    if type(value) != type({}):
        raise ValueError(name[0] + ' must be a dictionary with keys ' ' '.join(keys) +
                         ' whose values must be of type float')

    for key in keys:
        if not key in value.keys() or (not 'float' in str(type(value[key])) and not 'int' in str(type(value[key]))):
            raise ValueError(name[0] + ' must be a dictionary with keys ' ' '.join(
                keys) + ' whose values must be of type float')
        value[key] = float(value[key])


    if type(crs) != type('x'):
        raise ValueError('crs must be of type string')

    target_tile = geometry.Polygon([(bounds['xMin'],bounds['yMin']), (bounds['xMin'],bounds['yMax']), (bounds['xMax'],bounds['yMax']), (bounds['xMax'], bounds['yMin'])])
    sh = gpd.GeoDataFrame({'geometry':[target_tile]})    
    try:
        sh.crs = crs
    except:
        raise ValueError(name[0] + ' does not fit with the projection given by ' + name[1])

    return bounds, crs        

def validImage(name, value, required):
    if not required and type(value) == type(None):
        return

    if type(value) != type(np.zeros((2,2))):
        raise ValueError(name + ' must be a numpy array with either 2 or 3 dimensions')
    if len(value.shape) != 2  and len(value.shape) !=3 :
        raise ValueError(name + ' must be a numpy array with either 2 or 3 dimensions')
    if len(value.shape) ==3 and value.shape[2] !=3 and value.shape[2] !=1:
        raise ValueError(name + ' must have either 1 or 3 bands')

    return value

def validDataframe(name, value, required):
    if not required and type(value) == type(None):
        return

    if type(value) != type(pd.DataFrame()):
        raise ValueError(name + ' must be a pandas dataframe')
    value = value.copy()
    return value
        

def validNumpyArray(name, value, required):
    if not required and type(value) == type(None):
        return

    if type(value) != type(np.zeros((2,2) )):
        raise ValueError(name + 'must be a numpy array')

    return value


def validList(name, value, required):
    if not required and type(value) == type(None):
        return

    try:
        value = list(value)

    except:
        raise ValueError(name + ' must be an iterable')
    return value


def validGeoSeries(name, value, required, cores = 1):
    if not required and type(value) == type(None):
        return
    try:
        value = gpd.GeoDataFrame({'geometry':value})
    except:
        raise ValueError(name + ' must be a valid geoseries')
    value = validGeopandas(name, value, True, True)
    return value['geometry']


def validGeopandas(name, value, required, custom = False, cores = 1):
    if not required and type(value) == type(None):
        return

    if not custom and type(value) != type(gpd.GeoDataFrame()):
        raise ValueError(name + 'must be a geopandas dataframe')

    value = value.copy()        
    if str(type(value.crs)) == str(type(None)) and min(value.bounds['minx']) > -180  and max(value.bounds['maxx']) < 180 and min(value.bounds['miny']) > -90  and max(value.bounds['maxy']) < 90:
        print('assuming WGS84 coordinates for ' + name)
        value.crs = {'init': 'epsg:4326'}
    elif str(type(value.crs)) == str(type(None)):
        raise ValueError('Please provide CRS for ' + name)
    else:
        value = reprojectVector(features = value, targetEpsg = 4326, cores = cores)
        
    if 'id' in value.columns:
        del value['id']
    if 'userId' in value.columns:
        del value['userId']
    if 'attribution' in value.columns:
        del value['attribution']
    if 'radius' in value.columns:
        del value['radius']
    if 'color' in value.columns:
        del value['color']
        
    return(value)


def validBool(name, value, required):
    if not required and type(value) == type(None):
        return

    if not 'bool' in str(type(value)):
        raise ValueError(name + ' must be of type boolean')
    value = bool(value)
    return(value)


def validObject(name, value, required):
    if not required and type(value) == type(None):
        return


    try:
        json.dumps(value)
    except:
        raise ValueError(name + ' must be json serializable')
    return value

def validInt(name, value, required):
    if not required and type(value) == type(None):
        return

    if not 'int' in str(type(value)).lower():
        raise ValueError(name + ' must be of type int')

    value = int(value)
    return(value)


def validFloat(name, value, required):
    if not required and type(value) == type(None):
        return

    if not 'int' in str(type(value)).lower() and not 'float' in str(type(value)).lower():
        raise ValueError(name + ' must be of type float')

    value = float(value)
    return(value)


def validBounds(name, value, required):
    if not required and type(value) == type(None):
        return
    keys = ['xMin', 'xMax', 'yMin', 'yMax']
    if type(value) != type({}):
        raise ValueError(name + ' must be a dictionary with keys ' ' '.join(keys) +
                         ' whose values must be of type float')

    for key in keys:
        if not key in value.keys() or (not 'float' in str(type(value[key])) and not 'int' in str(type(value[key]))):
            raise ValueError(name + ' must be a dictionary with keys ' + ' '.join(
                keys) + ' whose values must be of type float')
        value[key] = float(value[key])


    if value['xMin'] >= value['xMax']:
        raise ValueError('xMax must be strictly larger than xMin')

    if value['yMin'] >= value['yMax']:
        raise ValueError('yMax must be strictly larger than yMin')

    return(value)


def validStringArray(name, value, required):
    if not required and type(value) == type(None):
        return

    try:
        value = list(value)        

    except:
        raise ValueError(name + ' must be an iterable')

    types = np.array([type(x) != type('x') for x in value])

    if np.sum(types) > 0:
        raise ValueError(name + ' must be a list of strings')

    return value


def validFloatArray(name, value, required):
    if not required and type(value) == type(None):
        return

    try:
        value = list(value)

    except:
        raise ValueError(name + ' must be an iterable')

    types = np.array([(not 'float' in str(type(x))) and (
        not 'int' in str(type(x))) for x in value])

    if np.sum(types) > 0:
        raise ValueError(name + ' must be a list of strings')

    value = [float(x) for x in value]

    return value


def validIntArray(name, value, required):
    if not required and type(value) == type(None):
        return

    try:
        value = list(value)

    except:
        raise ValueError(name + ' must be an iterable')

    types = np.array([not 'int' in str(type(x)).lower() for x in value])

    if np.sum(types) > 0:
        raise ValueError(name + ' must be a list of integers')

    value = [int(x) for x in value]

    return value


def validDate(name, value, required):

    if not required and type(value) == type(None):
        return

    if type(value) != type(datetime.datetime.now()):
        raise ValueError(name + ' must be of type datetime')

    value = value.strftime('%Y-%m-%dT%H:%M:%S')
    return value






import pandas as pd

from ellipsis import sanitize
from ellipsis import apiManager
from ellipsis.util.root import getActualExtent
from ellipsis.util.root import parseGlb
from ellipsis.util import loadingBar

import geopandas as gpd
import datetime
import math
from io import BytesIO
import  pandas as pd




def fetchPoints(pathId, timestampId, extent, token= None, epsg = 3857, zoom = None, showProgress = True):
    token = sanitize.validString('token', token, False)
    pathId = sanitize.validUuid('pathId', pathId, True)
    timestampId = sanitize.validUuid('timestampId', timestampId, True)
    extent = sanitize.validBounds('extent', extent, True)
    epsg = sanitize.validInt('epsg', epsg, True)
    zoom = sanitize.validInt('zoom', zoom, False)
    showProgress = sanitize.validBool('showProgress', showProgress, True)

    xMin = extent['xMin']
    yMin = extent['yMin']
    xMax = extent['xMax']
    yMax = extent['yMax']

    res = getActualExtent(xMin, xMax, yMin, yMax, 'EPSG:' + str(epsg))
    if res['status'] == '400':
        raise ValueError('Invalid epsg and extent combination')

    bounds = res['message']

    xMinWeb = bounds['xMin']
    yMinWeb = bounds['yMin']
    xMaxWeb = bounds['xMax']
    yMaxWeb = bounds['yMax']

    info = apiManager.get('/path/' + pathId, None, token)


    timestamps = info['pointCloud']['timestamps']
    all_timestamps = [item['id'] for item in timestamps]
    if not timestampId in all_timestamps:
        raise ValueError('given timestamp does not exist')

    timestamp = next(item for item in timestamps if item["id"] == timestampId)
    if timestamp['status'] != 'active':
        raise ValueError('timestamp is not active, please active timestamp before making queries to it')
    zoom_n = timestamp['zoom']

    if type(zoom) == type(None):
        zoom = zoom_n

    if zoom > zoom_n:
        raise ValueError('Maximum allowed zoom for this layer is ', str(zoom_n))

    LEN = 2.003751e+07

    x_start = 2**zoom * (xMinWeb + LEN) / (2* LEN)
    x_end = 2**zoom * (xMaxWeb + LEN) / (2* LEN)
    y_end = 2**zoom * (LEN - yMinWeb) / (2* LEN)
    y_start = 2**zoom * (LEN - yMaxWeb) / (2* LEN)

    x1_osm = math.floor(x_start)
    x2_osm = math.floor(x_end)
    y1_osm = math.floor(y_start)
    y2_osm = math.floor(y_end)
    dfs = []
    I = 0
    N = len(range(x1_osm, x2_osm+1)) * len(range(y1_osm, y2_osm+1))*5
    for tileX in range(x1_osm, x2_osm + 1):
        for tileY in range(y1_osm, y2_osm + 1):
            for p in range(5):
                body = {'zipTheResponse':True, 'page': p}
                r = apiManager.get('/path/' + pathId + '/pointCloud/timestamp/' + timestampId + '/tile/' + str(zoom) + '/' + str(tileX) + '/' + str(tileY), body, token, False)

                if r.status_code == 403:
                    raise ValueError('insufficient access')
                elif r.status_code == 204:
                    pass
                elif r.status_code != 200:
                    raise ValueError(r.text)
                else:
                    stream = BytesIO(r.content).read()
                    output = parseGlb(stream)
                    df_n = pd.DataFrame({'x':output['x'], 'y':output['y'], 'z':output['z'], 'red':output['red'], 'green':output['green'], 'blue':output['blue']   })
                    dfs = dfs + [df_n]

                if showProgress:
                    I = I + 1
                    loadingBar(I, N)


    if len(dfs) == 0:
        raise ValueError('No points found in this region. Did you specify the correct epsg code? Given epsg: ' + str(epsg))
    df = pd.concat(dfs)
    return df

def add(pathId, token, description= None, date ={'from': datetime.datetime.now(), 'to': datetime.datetime.now()}):
    

    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    date = sanitize.validDateRange('date', date, True)    
    description = sanitize.validString('description', description, False)    

    body = { 'date': date, 'description': description}    
    r = apiManager.post('/path/' + pathId + '/pointCloud/timestamp'  , body, token)
    return r
    
def edit(pathId, timestampId, token, date=None, description= None):

    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    timestampId = sanitize.validUuid('timestampId', timestampId, True)  
    description = sanitize.validString('description', description, False)    
    date = sanitize.validDateRange('date', date, False)    

    body = {'date':date, 'description': description}    
    r = apiManager.patch('/path/' + pathId + '/pointCloud/timestamp/' + timestampId  , body, token)
    return r

def getBounds(pathId, timestampId, token = None):
    token = sanitize.validString('token', token, False)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    timestampId = sanitize.validUuid('timestampId', timestampId, True)  
    r = apiManager.get('/path/' + pathId + '/pointCloud/timestamp/' + timestampId + '/bounds'  , None, token)
    r = {'id': 0, 'properties':{}, 'geometry':r}

    r  = gpd.GeoDataFrame.from_features([r])
    r = r.unary_union
    return r

def activate(pathId, timestampId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    timestampId = sanitize.validUuid('timestampId', timestampId, True)  
    r = apiManager.post('/path/' + pathId + '/pointCloud/timestamp/' + timestampId + '/activate'  , None, token)
    return r

def deactivate(pathId, timestampId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    timestampId = sanitize.validUuid('timestampId', timestampId, True)  
    r = apiManager.post('/path/' + pathId + '/pointCloud/timestamp/' + timestampId + '/deactivate'  , None, token)
    return r


def delete(pathId, timestampId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    timestampId = sanitize.validUuid('timestampId', timestampId, True)  

    r = apiManager.delete('/path/' + pathId + '/pointCloud/timestamp/' + timestampId  , None, token)
    return r


def trash(pathId, timestampId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    timestampId = sanitize.validUuid('timestampId', timestampId, True)  
    body = {'trashed' : True}
    r = apiManager.put('/path/' + pathId + '/pointCloud/timestamp/' + timestampId + '/trashed'  , body, token)
    return r

    
def recover(pathId, timestampId, token):
    token = sanitize.validString('token', token, True)
    pathId = sanitize.validUuid('pathId', pathId, True)  
    timestampId = sanitize.validUuid('timestampId', timestampId, True)  
    body = {'trashed' : False}
    r = apiManager.put('/path/' + pathId + '/pointCloud/timestamp/' + timestampId + '/trashed'  , body, token)
    return r
